@@ -7,6 +7,10 @@
 
 #pragma once
 
+#if __cplusplus < 201703L
+#error "embind requires -std=c++17 or newer"
+#endif
+
 #include <cassert>
 #include <cstddef>
 #include <functional>
@@ -14,9 +18,7 @@
 #include <string>
 #include <type_traits>
 #include <vector>
-#if __cplusplus >= 201703L
 #include <optional>
-#endif
 
 #include <emscripten/em_asm.h>
 #include <emscripten/val.h>
@@ -215,6 +217,12 @@ void _embind_register_class_class_property(
     const char* setterSignature,
     GenericFunction setter);
 
+void _embind_register_iterable(
+    TYPEID classType,
+    TYPEID elementType,
+    const char* sizeMethodName,
+    const char* getMethodName);
+
 EM_VAL _embind_create_inheriting_constructor(
     const char* constructorName,
     TYPEID wrapperType,
@@ -224,7 +232,8 @@ void _embind_register_enum(
     TYPEID enumType,
     const char* name,
     size_t size,
-    bool isSigned);
+    bool isSigned,
+    int policyValue);
 
 void _embind_register_smart_ptr(
     TYPEID pointerType,
@@ -258,6 +267,11 @@ void _embind_register_user_type(
     TYPEID type,
     const char* typeName);
 
+void _embind_register_user_type_definition(
+    TYPEID type,
+    const char* typeName,
+    const char* typeDefinition);
+
 // Register an InitFunc in the global linked list of init functions.
 void _embind_register_bindings(struct InitFunc* f);
 
@@ -266,7 +280,7 @@ void _embind_register_bindings(struct InitFunc* f);
 // to avoid static constructor ordering issues.
 struct InitFunc {
   InitFunc(void (*init_func)()) : init_func(init_func) {
-    // This the function immediately upon constructions, and also register
+    // This calls the function immediately upon construction, and also registers
     // it so that it can be called again on each worker that starts.
     init_func();
     _embind_register_bindings(this);
@@ -1579,6 +1593,19 @@ public:
         return *this;
     }
 
+    template<typename ElementType>
+    EMSCRIPTEN_ALWAYS_INLINE const class_& iterable(
+        const char* sizeMethodName,
+        const char* getMethodName) const {
+        using namespace internal;
+        _embind_register_iterable(
+            TypeID<ClassType>::get(),
+            TypeID<ElementType>::get(),
+            sizeMethodName,
+            getMethodName);
+        return *this;
+    }
+
     template<
         typename FieldType,
         typename... Policies,
@@ -1586,7 +1613,7 @@ public:
         // overload.
         typename = typename std::enable_if<
             !std::is_function<FieldType>::value &&
-            internal::conjunction<internal::isPolicy<Policies>...>::value>::type>
+            std::conjunction<internal::isPolicy<Policies>...>::value>::type>
     EMSCRIPTEN_ALWAYS_INLINE const class_& property(const char* fieldName, const FieldType ClassType::*field, Policies...) const {
         using namespace internal;
         using ReturnPolicy = GetReturnValuePolicy<FieldType, Policies...>::tag;
@@ -1614,7 +1641,7 @@ public:
         // overload.
         typename = typename std::enable_if<
             !std::is_function<FieldType>::value &&
-            internal::conjunction<internal::isPolicy<Policies>...>::value>::type>
+            std::conjunction<internal::isPolicy<Policies>...>::value>::type>
     EMSCRIPTEN_ALWAYS_INLINE const class_& property(const char* fieldName, FieldType ClassType::*field, Policies...) const {
         using namespace internal;
         using ReturnPolicy = GetReturnValuePolicy<FieldType, Policies...>::tag;
@@ -1643,7 +1670,7 @@ public:
         // Prevent the template from wrongly matching the getter/setter overload
         // of this function.
         typename = typename std::enable_if<
-            internal::conjunction<internal::isPolicy<Policies>...>::value>::type>
+            std::conjunction<internal::isPolicy<Policies>...>::value>::type>
     EMSCRIPTEN_ALWAYS_INLINE const class_& property(const char* fieldName, Getter getter, Policies...) const {
         using namespace internal;
 
@@ -1695,8 +1722,8 @@ public:
         auto ster = &SP::template set<ClassType>;
 
         typename WithPolicies<Policies...>::template ArgTypeList<typename GP::ReturnType> returnType;
-        // XXX: This currently applies all the polices (including return value polices) to the
-        // setter function argument to allow pointers. Using return value polices doesn't really
+        // XXX: This currently applies all the policies (including return value policies) to the
+        // setter function argument to allow pointers. Using return value policies doesn't really
         // make sense on an argument, but we don't have separate argument policies yet.
         typename WithPolicies<Policies...>::template ArgTypeList<typename SP::ArgumentType> argType;
 
@@ -1770,7 +1797,6 @@ public:
     }
 };
 
-#if __cplusplus >= 201703L
 template<typename T>
 void register_optional() {
     // Optional types are automatically registered for some internal types so
@@ -1785,7 +1811,6 @@ void register_optional() {
         internal::TypeID<std::optional<T>>::get(),
         internal::TypeID<typename std::remove_pointer<T>::type>::get());
 }
-#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 // VECTORS
@@ -1795,9 +1820,6 @@ namespace internal {
 
 template<typename VectorType>
 struct VectorAccess {
-// This nearly duplicated code is used for generating more specific TypeScript
-// types when using more modern C++ versions.
-#if __cplusplus >= 201703L
     static std::optional<typename VectorType::value_type> get(
         const VectorType& v,
         unsigned int index
@@ -1808,18 +1830,6 @@ struct VectorAccess {
             return {};
         }
     }
-#else
-    static val get(
-        const VectorType& v,
-        unsigned int index
-    ) {
-        if (index < v.size()) {
-            return val(v[index], allow_raw_pointers());
-        } else {
-            return val::undefined();
-        }
-    }
-#endif
 
     static bool set(
         VectorType& v,
@@ -1855,9 +1865,9 @@ struct VectorAccess {
 template<typename T, class Allocator=std::allocator<T>>
 class_<std::vector<T, Allocator>> register_vector(const char* name) {
     typedef std::vector<T, Allocator> VecType;
-#if __cplusplus >= 201703L
     register_optional<T>();
-#endif
+    using VectorElementType =
+        typename internal::RawPointerTransformer<T, std::is_pointer<T>::value>::type;
 
     return class_<VecType>(name)
         .template constructor<>()
@@ -1866,7 +1876,7 @@ class_<std::vector<T, Allocator>> register_vector(const char* name) {
         .function("size", internal::VectorAccess<VecType>::size, allow_raw_pointers())
         .function("get", internal::VectorAccess<VecType>::get, allow_raw_pointers())
         .function("set", internal::VectorAccess<VecType>::set, allow_raw_pointers())
-        ;
+        .template iterable<VectorElementType>("size", "get");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1877,9 +1887,6 @@ namespace internal {
 
 template<typename MapType>
 struct MapAccess {
-// This nearly duplicated code is used for generating more specific TypeScript
-// types when using more modern C++ versions.
-#if __cplusplus >= 201703L
     static std::optional<typename MapType::mapped_type> get(
         const MapType& m,
         const typename MapType::key_type& k
@@ -1891,19 +1898,6 @@ struct MapAccess {
             return i->second;
         }
     }
-#else
-    static val get(
-        const MapType& m,
-        const typename MapType::key_type& k
-    ) {
-        auto i = m.find(k);
-        if (i == m.end()) {
-            return val::undefined();
-        } else {
-            return val(i->second);
-        }
-    }
-#endif
 
     static void set(
         MapType& m,
@@ -1935,9 +1929,7 @@ template<typename K, typename V, class Compare = std::less<K>,
     class Allocator = std::allocator<std::pair<const K, V>>>
 class_<std::map<K, V, Compare, Allocator>> register_map(const char* name) {
     typedef std::map<K,V, Compare, Allocator> MapType;
-#if __cplusplus >= 201703L
     register_optional<V>();
-#endif
 
     return class_<MapType>(name)
         .template constructor<>()
@@ -1948,36 +1940,7 @@ class_<std::map<K, V, Compare, Allocator>> register_map(const char* name) {
         ;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// std::optional
-////////////////////////////////////////////////////////////////////////////////
 
-#if __cplusplus >= 201703L
-namespace internal {
-template <typename T>
-struct BindingType<std::optional<T>> {
-    using ValBinding = BindingType<val>;
-    using WireType = ValBinding::WireType;
-
-    template<typename ReturnPolicy = void>
-    static WireType toWireType(std::optional<T> value, rvp::default_tag) {
-        if (value) {
-            return ValBinding::toWireType(val(*value, allow_raw_pointers()), rvp::default_tag{});
-        }
-        return ValBinding::toWireType(val::undefined(), rvp::default_tag{});
-    }
-
-
-    static std::optional<T> fromWireType(WireType value) {
-        val optional = val::take_ownership(value);
-        if (optional.isUndefined()) {
-            return {};
-        }
-        return optional.as<T>();
-    }
-};
-} // end namespace internal
-#endif
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1989,13 +1952,14 @@ class enum_ {
 public:
     typedef EnumType enum_type;
 
-    enum_(const char* name) {
+    enum_(const char* name, enum_value_type valueType = enum_value_type::object) {
         using namespace internal;
         _embind_register_enum(
             internal::TypeID<EnumType>::get(),
             name,
             sizeof(EnumType),
-            std::is_signed<typename std::underlying_type<EnumType>::type>::value);
+            std::is_signed<typename std::underlying_type<EnumType>::type>::value,
+            static_cast<int>(valueType));
     }
 
     enum_& value(const char* name, EnumType value) {
@@ -2044,6 +2008,12 @@ inline void register_type(const char* name) {
   _embind_register_user_type(TypeID<T>::get(), name);
 }
 
+template <typename T>
+inline void register_type(const char* name, const char* definition) {
+  using namespace internal;
+  _embind_register_user_type_definition(TypeID<T>::get(), name, definition);
+}
+
 // EMSCRIPTEN_BINDINGS creates a static struct to initialize the binding which
 // will get included in the program if the translation unit in which it is
 // defined gets linked into the program. Using a C++ constructor here ensures it
