@@ -18,6 +18,7 @@ import subprocess
 import sys
 
 WINDOWS = sys.platform.startswith('win')
+MSYS2 = 'MSYSTEM' in os.environ
 
 EXCLUDES = [os.path.normpath(x) for x in '''
 test/third_party
@@ -66,9 +67,9 @@ def copy_emscripten(target):
   excludes = EXCLUDES
   # We have a few launcher scripts that are checked into git still.
   # Exclude the ones not designed for the current platforms.
-  if WINDOWS:
+  if WINDOWS and not MSYS2:
     excludes += [os.path.splitext(l)[0] for l in LAUNCHER_BAT_SCRIPTS]
-  else:
+  elif not MSYS2:
     excludes += LAUNCHER_BAT_SCRIPTS
 
   os.chdir(emscripten_root)
