@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-"""Find entries in struct_info.json that are not needd by
+"""Find entries in struct_info.json that are not needed by
 any JS library code and can be removed."""
 
 import json
@@ -19,7 +19,7 @@ from tools import utils
 def check_structs(info):
   for struct, values in info['structs'].items():
     key = 'C_STRUCTS\\.' + struct + '\\.'
-    # grep --quiet ruturns 0 when there is a match
+    # grep --quiet returns 0 when there is a match
     if subprocess.run(['git', 'grep', '--quiet', key], check=False).returncode != 0:
       print(key)
     else:
@@ -34,7 +34,7 @@ def check_structs(info):
 def check_defines(info):
   for define in info['defines']:
     key = r'cDefs\.' + define + r'\>'
-    # grep --quiet ruturns 0 when there is a match
+    # grep --quiet returns 0 when there is a match
     if subprocess.run(['git', 'grep', '--quiet', key], check=False).returncode != 0:
       print(define)
 
